' Program.vb : main project file.
'!
'** NAME: \file  Program.vb              									\n
'**																			\n
'** (C) Copyright PREHKEYTEC 2018.      								    \n
'**																			\n
'** PURPOSE:																\n
'**  This module contains routines for C# PKTUSB API demonstration.			\n
'**																			\n
'** No Warranty																\n
'** The Software is being delivered to you "AS IS" and PrehKeyTec GmbH		\n
'** makes no warranty as to its use or performance. Details see readme.		\n
'**																			\n
'** The PKTUSBAPI is written in c# (managed code).							\n
'**																			\n
'**  \brief This console C# demo works with PKTUSBAPI.			   	        \n
'**																		   	\n
'** FUNCTIONS:																\n
'**  Main routine													        \n
'**  OpenDevice 													        \n
'**  OpenDeviceOCR													        \n
'**  myAPI_OnDeviceStatusChanged											\n
'**	 myDevice_OnKeyLockChanged												\n
'**  myDevice_OnReceivedPOSkey												\n
'**	 myDevice_OnReceivedMSR  												\n
'**	 myDevice_OnReceivedOCR  												\n
'**																			\n
'** \author          B Grossmannn      	\version       5.1				    \n
'**																			\n
'**   									\date   	   19 Jan 2018			\n
'**																			\n
'** REVISION HISTORY:														\n
'**																			\n
'**  DATE       	VERSION	PROGRAMMER     	DESCRIPTION							\n
'**  ========== 	======= ============== 	=======================		        \n
'**  19. Jan 18    5.1  	Grossmann B.   	Module created 						\n
'


' INCLUDES

#Const MY_OCR_DEVICE = 1

Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Threading
Imports System.ComponentModel

' Reference to PKTUSBAPI.DLL The Path to the DLL should be relative to Demoapplication
' With a post built command Xcopies the required DLL from Demoapplication to target folders
Imports PKTUSBAPI
'Reference to PKTUSBAPI.DLL



Class Program
	Private Shared myAPI As PKTUSBAPI_Class = Nothing
	Private Shared myDevice As USB_Device = Nothing
	' define MY_OCR_DEVICE in the first line of this source to enable OCR support for the OCR MCI111A keyboard or OCR ML2A module.
	#If MY_OCR_DEVICE Then
	Private Shared myOCRDevice As USB_Device = Nothing
	#End If
	Friend Shared Sub Main(args As String())
		Try
			'Initialize the PKTUSBAPI (Show all existing PrehKeyTec USB Devices) 
			Console.WriteLine(vbLf & "Initialize the PKTUSBAPI")
			myAPI = New PKTUSBAPI_Class(Nothing)

			'Show the library versions 
			Console.WriteLine("PKTUSBAPI.dll Version: " & myAPI.VersionAPI)
			Console.WriteLine("PKTUSBxx.dll Version: " & myAPI.VersionDLL)

			'Name of my log file (text file)
			Dim LogFileName As [String] = "MyLogFile.txt"

			'Log only errors, warnings and inforamtion
			Dim LogLevel As Integer = Constants.LOG_LEVEL_ERROR_WARNING_INFO

			'Append new messages to the log file if it already exists
			Dim LogAppend As Boolean = True

			'Activate log file
			Dim ret As Integer = myAPI.StartLog(LogFileName, LogAppend, LogLevel)
			If ret = ErrorStatus.Succeeded Then
				Console.WriteLine("Logging successful started")
			Else
				Console.WriteLine("Starting logging failed")
			End If

			' Open already connected device
			OpenDevice()
			#If MY_OCR_DEVICE Then
			' Open already connected OCR device
			OpenDeviceOCR()
			#End If
			'Set up on device status changed event
			Console.WriteLine(vbLf & "Set up Event Handler: OnDeviceStatusChanged")
			AddHandler myAPI.OnDeviceStatusChanged, New EventHandler(Of EventArguments.OnDeviceStatusChanged)(AddressOf myAPI_OnDeviceStatusChanged)
				' wait for any key to quit
			Console.ReadKey()
		Catch e As OverflowException
		End Try

	End Sub

	Private Shared Sub OpenDevice()
        'Open the device that was found first from the API with the usage page of 0xfffa
        Const UsagePage As UShort = &HFFFA
        myDevice = myAPI.OpenDevice(UsagePage)
		If myDevice IsNot Nothing Then
			'Display some device information
			Console.WriteLine(vbLf & "+  +  +  +  +  +  +  +  +  +  +  +  +  +")
			Console.WriteLine("Device opened:")
			Console.WriteLine(myDevice.DevicePath & vbLf)
			Console.WriteLine(myDevice.VersionString)

			'Play 'Beep' tone on it
			myDevice.PlayBeep()

			'Set up on received MSR data event for my device
			Console.WriteLine(vbLf & "Set up Event Handler: OnReceivedMSR")
			AddHandler myDevice.OnReceivedMSR, New EventHandler(Of EventArguments.OnReceivedMSR)(AddressOf myDevice_OnReceivedMSR)

			'Set up on received POS key event for my device
			Console.WriteLine("Set up Event Handler: OnReceivedPOSkey")
			AddHandler myDevice.OnReceivedPOSkey, New EventHandler(Of EventArguments.OnReceivedPOSkey)(AddressOf myDevice_OnReceivedPOSkey)

			'Set up on received key lock changed event for my device
			Console.WriteLine("Set up Event Handler: OnReceivedKeyLockChanged")
			AddHandler myDevice.OnKeyLockChanged, New EventHandler(Of EventArguments.OnKeyLockChanged)(AddressOf myDevice_OnKeyLockChanged)

			Console.WriteLine("+  +  +  +  +  +  +  +  +  +  +  +  +  +")
		End If
	End Sub


	#If MY_OCR_DEVICE Then
	Private Shared Sub OpenDeviceOCR()
        'Open the device that was found first from the API with the usage page of 0xfff9 (OCR)
        Const UsagePage As UShort = &HFFF9
        myOCRDevice = myAPI.OpenDevice(UsagePage)
		If myOCRDevice IsNot Nothing Then
			'Display some device information
			Console.WriteLine(vbLf & "+  +  +  +  +  +  +  +  +  +  +  +  +  +")
			Console.WriteLine("OCR Device opened:")
			Console.WriteLine(myOCRDevice.DevicePath & vbLf)
			Console.WriteLine(myOCRDevice.VersionString)

			'Set up on received MSR data event for my device
			Console.WriteLine(vbLf & "Set up Event Handler: OnReceivedMSR")
			AddHandler myOCRDevice.OnReceivedMSR, New EventHandler(Of EventArguments.OnReceivedMSR)(AddressOf myDevice_OnReceivedMSR)

			'Set up on received OCR data event for my device
			Console.WriteLine("Set up Event Handler: OnReceivedOCR")
			AddHandler myOCRDevice.OnReceivedOCR, New EventHandler(Of EventArguments.OnReceivedOCR)(AddressOf myDevice_OnReceivedOCR)

			Console.WriteLine("+  +  +  +  +  +  +  +  +  +  +  +  +  +")
		End If
	End Sub
	#End If




	Private Shared Sub myAPI_OnDeviceStatusChanged(sender As Object, e As EventArguments.OnDeviceStatusChanged)
		'Do something only if a device was attached or detached
		If e.ConnectedStatusChanged Then
			'Check the usage page of the de-/attached device to see if it is a PrehKeyTec standard device
			If e.ChangedDevice.UsagePage = &Hfffa Then
				'Check if myDevice was detached
				If myDevice IsNot Nothing AndAlso e.ChangedDevice.DevicePath.Equals(myDevice.DevicePath) AndAlso Not e.ChangedDevice.IsConnected Then
					'Close myDevice instance
					Console.WriteLine(vbLf & "Device closed")
					myDevice.Close()
					' device close
					myDevice = Nothing
				End If

				'Searching for a device to open if myDevice is 'null'
				If myDevice Is Nothing Then
						' try to open new device
					OpenDevice()
				End If
			End If


			#If MY_OCR_DEVICE Then
			'Check the usage page of the de-/attached device to see if it is a PrehKeyTec OCR device
			If e.ChangedDevice.UsagePage = &Hfff9 Then
				'Check if myOCRDevice was detached
				If myOCRDevice IsNot Nothing AndAlso e.ChangedDevice.DevicePath.Equals(myOCRDevice.DevicePath) AndAlso Not e.ChangedDevice.IsConnected Then
					'Close myOCRDevice instance
					Console.WriteLine(vbLf & "OCR Device closed")
					myOCRDevice.Close()
					' close OCR device
					myOCRDevice = Nothing
				End If

				'Searching for an OCR device to open if myOCRDevice is 'null'
				If myOCRDevice Is Nothing Then
						' try to open OCR device
					OpenDeviceOCR()
				End If
				#End If
			End If
		End If
	End Sub

	Private Shared Sub myDevice_OnKeyLockChanged(sender As Object, e As EventArguments.OnKeyLockChanged)
		'Show received event data 
		Console.WriteLine("[KeyLock] Position: " & e.Position)
		' get and print Keylock position
	End Sub

	Private Shared Sub myDevice_OnReceivedPOSkey(sender As Object, e As EventArguments.OnReceivedPOSkey)
		'Show received event data 
		Console.WriteLine("[POS Key] Key: " & e.Key & " Status: " & (If(e.Pressed, "pressed", "released")))
		' get and print POSkey number and status
	End Sub

	Private Shared Sub myDevice_OnReceivedMSR(sender As Object, e As EventArguments.OnReceivedMSR)
		'Show received event data 
		Console.WriteLine("[MSR]")
		Console.WriteLine("Track 1: " & e.MSRtrack1)
		' get and print MSR line
		Console.WriteLine("Track 2: " & e.MSRtrack2)
		Console.WriteLine("Track 3: " & e.MSRtrack3)
	End Sub

	#If MY_OCR_DEVICE Then
	Private Shared Sub myDevice_OnReceivedOCR(sender As Object, e As EventArguments.OnReceivedOCR)
		'Show received event data 
		Console.WriteLine("[OCR]")
		Console.WriteLine("MRZ line 1: " & e.MRZline1)
		' get and print OCR line
		Console.WriteLine("MRZ line 2: " & e.MRZline2)
		Console.WriteLine("MRZ line 3: " & e.MRZline3)
	End Sub
	#End If
End Class
